/* Colors.h provides a Color hierarchy.
 * ...
 */
 
#ifndef COLORS
#define COLORS

#include <iostream>  // istream, ostream
#include <string>    // string
using namespace std;

// ***** Color -- the superclass ************************
class Color
{
 public:
   Color(const string& name);
   
   string asString() const;
   static Color* fromString(const string& name);
   
   virtual double wavelength() const = 0;
   double frequency() const;
   virtual Color* next()       const = 0;
//   previous() left as an exercise
   
   friend bool operator==(const Color& left, const Color& right);
   friend bool operator!=(const Color& left, const Color& right);
   friend bool operator< (const Color& left, const Color& right);
   friend bool operator> (const Color& left, const Color& right);
   friend bool operator<=(const Color& left, const Color& right);
   friend bool operator>=(const Color& left, const Color& right);
   
 private:
   string myName;
};

// **** Red subclass **************************************
class Red : public Color
{
 public:
   Red();
   virtual double wavelength() const;
   virtual Color* next() const;
};

// **** Orange sublass **************************************
class Orange : public Color
{
 public:
   Orange();
   virtual double wavelength() const;
   virtual Color* next() const;
};

// **** Yellow sublass **************************************
class Yellow : public Color
{
 public:
   Yellow();
   virtual double wavelength() const;
   virtual Color* next() const;
};

// **** Green subclass **************************************
class Green : public Color
{
 public:
   Green();
   virtual double wavelength() const;
   virtual Color* next() const;
};

// **** Blue subclass **************************************
class Blue : public Color
{
 public:
   Blue();
   virtual double wavelength() const;
   virtual Color* next() const;
};

// **** Indigo subclass **************************************
class Indigo : public Color
{
 public:
   Indigo();
   virtual double wavelength() const;
   virtual Color* next() const;
};


// **** Violet subclass **************************************
class Violet : public Color
{
 public:
   Violet();
   virtual double wavelength() const;
   virtual Color* next() const;
};

// ***** Operations *******************************************
// ***** Color operations *************************************
// ---- Color-to-string converter ------
inline string Color::asString() const { return myName; }

// ---- frequency method ----------------
const double SPEED_OF_LIGHT = 2.998e8;
inline double Color::frequency() const { return SPEED_OF_LIGHT / wavelength(); }

// ---- Input Function -----------------
istream& operator>>(istream& in, Color*& colorPtr);

// ---- Output Function ----------------
ostream& operator<<(ostream& out, const Color*& colorPtr);

// ---- Equality ------------------------
inline bool operator==(const Color& left, const Color& right)
{
  return left.asString() == right.asString();
}


// ---- Inquality ------------------------
inline bool operator!=(const Color& left, const Color& right)
{
  return left.asString() != right.asString();
}

// ---- Less-than ------------------------
inline bool operator<(const Color& left, const Color& right)
{
  return left.asString() < right.asString();
}


// ---- greater-than ------------------------
inline bool operator>(const Color& left, const Color& right)
{
  return left.asString() > right.asString();
}

// ---- Less-than-or-equals ------------------------
inline bool operator<=(const Color& left, const Color& right)
{
  return left.asString() <= right.asString();
}

// ---- greater-than-or-equals ------------------------
inline bool operator>=(const Color& left, const Color& right)
{
  return left.asString() >= right.asString();
}

// ***** Red operations *************************************
inline Red::Red() : Color("Red") {}

inline double Red::wavelength() const { return 6.5e-7; }

inline Color* Red::next() const { return new Orange(); }


// ***** Orange operations *************************************
inline Orange::Orange() : Color("Orange") {}

inline double Orange::wavelength() const { return 6.0e-7; }

inline Color* Orange::next() const { return new Yellow(); }


// ***** Yellow operations *************************************
inline Yellow::Yellow() : Color("Yellow") {}

inline double Yellow::wavelength() const { return 5.8e-7; }

inline Color* Yellow::next() const { return new Green(); }


// ***** Green operations *************************************
inline Green::Green() : Color("Green") {}

inline double Green::wavelength() const { return 5.2e-7; }

inline Color* Green::next() const { return new Blue(); }


// ***** Blue operations *************************************
inline Blue::Blue() : Color("Blue") {}

inline double Blue::wavelength() const { return 4.7e-7; }

inline Color* Blue::next() const { return new Indigo(); }


// ***** Indigo operations *************************************
inline Indigo::Indigo() : Color("Indigo") {}

inline double Indigo::wavelength() const { return 4.4e-7; }

inline Color* Indigo::next() const { return new Violet(); }


// ***** Violet operations *************************************
inline Violet::Violet() : Color("Violet") {}

inline double Violet::wavelength() const { return 4.1e-7; }

inline Color* Violet::next() const { return NULL; }

#endif
